#!/usr/bin/env python3
"""
souleyez.ui - User interface components
"""

from .dashboard import run_dashboard
from .interactive import run_interactive_menu

__all__ = ["run_interactive_menu", "run_dashboard"]
